@@ -34,7 +34,7 @@
 #include "mxBeeBase.h"
 
 /* Version number: Major.Minor.Patchlevel */
-#define MXBEEBASE_VERSION "3.2.6"
+#define MXBEEBASE_VERSION "3.2.7"
 
 /* Define these to have the module use free lists (saves malloc calls) */
 /*#define MXBEEINDEX_FREELIST*/
@@ -1913,6 +1913,10 @@ MX_EXPORT(void)
     if (moddict == NULL)
 	goto onError;
     insobj(moddict,"__version__",PyString_FromString(MXBEEBASE_VERSION));
+    insobj(moddict,"sizeof_bNode",PyInt_FromLong(sizeof(bNode)));
+    insobj(moddict,"sizeof_bKey",PyInt_FromLong(sizeof(bKey)));
+    insobj(moddict,"sizeof_bRecAddr",PyInt_FromLong(sizeof(bRecAddr)));
+    insobj(moddict,"sizeof_bIdxAddr",PyInt_FromLong(sizeof(bIdxAddr)));
 
     /* Errors */
     if (!(mxBeeIndex_Error = insexc(moddict,"BeeIndexError")))
