@@ -9,17 +9,18 @@
     It is safe to do a 'from mx.Log import *'.
 
     Copyright (c) 1999-2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 
 """
 # Be careful with these symbols since we want 'from mx.Log import *'
 # to not clutter up the users namespace.
-import sys,string,types,traceback
-from mx import DateTime,Tools
-_sys,_string,_DateTime,_types,_traceback = sys,string,DateTime,types,traceback
-del sys,string,DateTime,types,traceback
+import sys, types, traceback, os
+from mx import DateTime, Tools
+(_sys,_DateTime,_types,_traceback,_os) = \
+    (sys,DateTime,types,traceback,os)
+del sys,DateTime,types,traceback,os
 
 # Load new builtins
 import  mx.Tools.NewBuiltins
@@ -56,6 +57,27 @@ _NON_RECURSIVE = (_types.ModuleType,
                   _types.BuiltinMethodType,
                   _types.ClassType,
                   __builtins__)
+_STRING_TYPES = (_types.StringType,
+                 _types.UnicodeType)
+
+def is_string(obj):
+
+    """ Return 1/0 depending on whether obj is an instance of
+        a string type.
+
+    """
+    return isinstance(obj, _STRING_TYPES)
+
+# For Python < 2.2, isinstance() does not accept tuples of types, so
+# override is_string() with a version that does not need this.
+try:
+    isinstance('', _STRING_TYPES)
+except TypeError:
+    def is_string(obj):
+        for typeobj in _STRING_TYPES:
+            if isinstance(obj, typeobj):
+                return 1
+        return 0
 
 def print_tb_locals(tb=None, *args, **kws):
 
@@ -66,6 +88,7 @@ def print_tb_locals(tb=None, *args, **kws):
         tb = _sys.exc_info()[2]
     while tb.tb_next != None:
         tb = tb.tb_next
+
     # Show the local variables
     apply(print_frame_locals, (tb.tb_frame,) + args, kws)
 
@@ -78,6 +101,7 @@ def print_tb_globals(tb=None, *args, **kws):
         tb = _sys.exc_info()[2]
     while tb.tb_next != None:
         tb = tb.tb_next
+
     # Show the global variables
     apply(print_frame_globals, (tb.tb_frame,) + args, kws)
 
@@ -97,6 +121,7 @@ def print_tb_vars(tb=None, *args, **kws):
     if tb.tb_frame.f_locals is not tb.tb_frame.f_globals:
         # Show the local variables
         apply(print_frame_locals, (tb.tb_frame,) + args, kws)
+
     # Show the global variables
     apply(print_frame_globals, (tb.tb_frame,) + args, kws)
 
@@ -107,8 +132,7 @@ def filter_private_attributes(key):
         For use as filter argument to one of the print_xxx() APIs.
     
     """
-    if type(key) is _types.StringType and \
-       key[:1] == '_':
+    if is_string(key) and key[:1] == '_':
         return 0
     return 1
 
@@ -174,11 +198,14 @@ def print_dict(dict,file,indent='',levels=1,reprkeys=0,
             r = repr(v)
         except:
             r = '*repr()-error*'
+
         # Truncate
         if len(n) > _NAME_LEN_LIMIT:
             n = n[:_NAME_LEN_LIMIT] + '...'
         if len(r) > _VALUE_LEN_LIMIT:
             r = r[:_VALUE_LEN_LIMIT] + '...'
+
+        # Write item
         if reprkeys:
             file.write('%s %-15s = %s\n' % (indent,n,r))
         else:
@@ -188,9 +215,7 @@ def print_dict(dict,file,indent='',levels=1,reprkeys=0,
                             nonrecursive=nonrecursive, filter=filter)
 
 def print_sequence(obj,file=_sys.stdout,indent='',levels=2,
-                   nonrecursive=(),
-
-                   StringType=_types.StringType):
+                   nonrecursive=()):
 
     l = []
     unfold = 0
@@ -198,7 +223,8 @@ def print_sequence(obj,file=_sys.stdout,indent='',levels=2,
         length = len(obj)
     except (AttributeError, ValueError, TypeError):
         return
-    for i in trange(min(length,_VALUE_LEN_LIMIT)):
+    
+    for i in Tools.trange(min(length,_VALUE_LEN_LIMIT)):
         try:
             value = obj[i]
         except:
@@ -207,14 +233,23 @@ def print_sequence(obj,file=_sys.stdout,indent='',levels=2,
             r = repr(value)
         except:
             r = '*repr()-error*'
+
         # Truncate
         if len(r) > _VALUE_LEN_LIMIT:
             r = r[:_VALUE_LEN_LIMIT] + '...'
+
+        # Write value
         l.append((value,r))
-        if type(value) != StringType or len(value) > 1:
+
+        # Only unfold sequences that have non-string items or string items
+        # with more than on character
+        if not is_string(value) or len(value) > 1:
             unfold = 1
+            
     if len(obj) > _VALUE_LEN_LIMIT:
         l.append(('...','...truncated...'))
+
+    # Unfold value object
     if unfold:
         for i,(value,rvalue) in irange(l):
             file.write('%s%-15s = %s\n' % (indent, '[%i]' % i, rvalue))
@@ -223,9 +258,7 @@ def print_sequence(obj,file=_sys.stdout,indent='',levels=2,
                                 nonrecursive=nonrecursive)
 
 def print_recursive(obj,file=_sys.stdout,indent='',levels=1,
-                    nonrecursive=(),filter=None,
-
-                    StringType=_types.StringType):
+                    nonrecursive=(),filter=None):
 
     # Filter out nonrecursive types and objects
     try:
@@ -242,12 +275,15 @@ def print_recursive(obj,file=_sys.stdout,indent='',levels=1,
        obj.__dict__ is not None:
         print_dict(obj.__dict__,file,indent,levels,
                    nonrecursive=nonrecursive, filter=filter)
+
     elif hasattr(obj,'items'):
         print_dict(obj,file,indent,levels,1,
                    nonrecursive=nonrecursive, filter=filter)
-    elif issequence(obj) and not type(obj) == StringType:
+
+    elif Tools.issequence(obj) and not is_string(obj):
         print_sequence(obj,file,indent,levels,
                        nonrecursive=nonrecursive)
+
     elif hasattr(obj,'__members__'):
         d = {}
         for attr in obj.__members__:
@@ -262,9 +298,12 @@ def print_obj(obj,file=_sys.stdout,indent='',levels=2,
         r = repr(obj)
     except:
         r = '*repr()-error*'
+
     # Truncate
     if len(r) > _VALUE_LEN_LIMIT:
         r = r[:_VALUE_LEN_LIMIT] + '...'
+
+    # Write object representation
     file.write('%s%s\n' % (indent,r))
     print_recursive(obj, file, indent + '  ', levels-1, filter=filter)
 
@@ -283,26 +322,30 @@ def print_stack(file=_sys.stdout,levels=100,offset=0,locals=0):
         for i in range(offset + 1):
             if f.f_back is not None:
                 f = f.f_back
+
+    # Extract frames
     frames = []
     while f:
         frames.append(f)
         f = f.f_back
     frames.reverse()
+
     # Prepare stack
     stack = _traceback.extract_stack()
+
     # Make output
     file.write('Stack:\n')
     for (frame,(filename, lineno, name, line)) in \
-        tuples(frames,stack)[-levels:]:
+            Tools.tuples(frames,stack)[-levels:]:
         file.write(' File "%s", line %d, in %s\n' % (filename,lineno,name))
         if line:
-            file.write('  %s\n' % _string.strip(line))
+            file.write('  %s\n' % line.strip())
         if locals:
             print_frame_locals(frame,file,indent='   |',title='')
 
 def print_loaded_modules(file=_sys.stdout,
 
-                         string=_string,sys=_sys):
+                         sys=_sys):
 
     """ Print a listing of currently loaded modules to file.
     """
@@ -310,14 +353,14 @@ def print_loaded_modules(file=_sys.stdout,
     l.sort()
     file.write('Loaded modules and packages:\n')
     for k,v in l:
-        p = string.split(k,'.')
+        p = k.split('.')
         for i in range(len(p)-1):
             p[i] = '   '
-        n = string.join(p,'')
+        n = ''.join(p)
         if v is not None:
             if hasattr(v,'__path__'):
                 # package
-                file.write(' %s[%s]\n' % (string.join(p[:-1],''),p[-1]))
+                file.write(' %s[%s]\n' % (''.join(p[:-1]),p[-1]))
             else:
                 # module
                 file.write(' %s\n' % (n))
@@ -345,16 +388,49 @@ def format_traceback(title='### Dump of local variables:\n',
 
 class Log:
 
-    """ Log class.
-    """
-    ignore_level = SYSTEM_LOG_EVERYTHING        # log level
-    log_file = 'default.log'                    # name of the log file
-    log_id = None                               # log id to use
-    open_log_file = None                        # open file object
-    line_buffer = ''                            # line buffer used by .write()
+    """ Log class
     
-    # Attributes needed for file interface:
-    closed = 0                                  # We can always accept data
+    """
+
+    # Log ignore level. Log messages with an errno >= this ignore
+    # level are not logged
+    ignore_level = SYSTEM_LOG_EVERYTHING
+
+    # Log file to use. This can be a pathname or 'stderr'/'stdout' to
+    # log to the special file descriptors stderr/stdout.
+    log_file = 'stderr'
+
+    # Log ID to use in the log output. Default is set in .__init__()
+    log_id = None
+
+    # Open file-like object. This is opened on demand and can be closed
+    # via .close()
+    open_log_file = None
+
+    # Line buffer used by .write()
+    line_buffer = ''
+    
+    ### Attributes needed for file interface
+
+    # We can always accept data via .write(). Note that this variable
+    # is not changed by calling .close().
+    closed = 0
+
+    ### Convenience attributes
+
+    # Log levels as attributes for convenience; you can then write e.g.
+    # log(log.INFO, ...)
+    ALWAYS_LOG = SYSTEM_ALWAYS_LOG
+    FATAL = SYSTEM_FATAL
+    PANIC = SYSTEM_PANIC
+    ERROR = SYSTEM_ERROR
+    CANCEL = SYSTEM_CANCEL
+    IMPORTANT = SYSTEM_IMPORTANT
+    WARNING = SYSTEM_WARNING
+    MESSAGE = SYSTEM_MESSAGE
+    UNIMPORTANT = SYSTEM_UNIMPORTANT
+    INFO = SYSTEM_INFO
+    DEBUG = SYSTEM_DEBUG
     
     def __init__(self,ignore_level=None,log_file='stderr',
                  log_id=None,log_level=None):
@@ -379,15 +455,18 @@ class Log:
             self.ignore_level = ignore_level
         elif log_level is not None:
             self.ignore_level = log_level + 1
+        self.format_log_id(log_id)
         self.log_file = log_file
-        import os
-        if not log_id:
-            self.log_id = 'p' + str(os.getpid())
-        else:
-            self.log_id = log_id + ':' + str(os.getpid())
         self.orig_stderr = _sys.stderr
         self.orig_stdout = _sys.stdout
 
+    def format_log_id(self, log_id=None):
+
+        if not log_id:
+            self.log_id = 'p%i' % _os.getpid()
+        else:
+            self.log_id = '%s:%i' % (log_id, _os.getpid())
+
     def redirect_stderr(self):
 
         """ Redirects stderr output to the log mechanism.
@@ -435,6 +514,18 @@ class Log:
         self.open_log_file = file
         return file
 
+    def close(self):
+
+        """ Close the log file.
+
+            Note: The next logging request will automatically open the
+            file again.
+
+        """
+        if self.open_log_file:
+            self.open_log_file.close()
+        self.open_log_file = None
+
     def setup(self,ignore_level=None,log_file=None,log_id=None,
               log_level=None):
 
@@ -450,8 +541,7 @@ class Log:
             self.log_file = log_file
             self.open_log_file = None
         if log_id is not None:
-            import os
-            self.log_id = log_id + ':' + str(os.getpid())
+            self.format_log_id(log_id)
 
     def __call__(self,errno,*args):
 
@@ -564,7 +654,7 @@ class Log:
             s = ''
         f = self.open_log_file or self.open()
         f.write("%04i %s [%s]\n" % (errno,_DateTime.now(),self.log_id))
-        f.write(79*'-'+ '\n%s\n' % _string.strip(s) + 79*'-'+'\n')
+        f.write(79*'-'+ '\n%s\n' % s.strip() + 79*'-'+'\n')
         f.flush()
 
     def call(self,errno,level=1):
@@ -603,7 +693,7 @@ class Log:
         f = self.open_log_file or self.open()
         f.write("%04i %s [%s] %s\n" %
                 (errno,_DateTime.now(),self.log_id, errorname))
-        f.write(79*'-'+ '\n%s\n' % _string.strip(str(s)) + 79*'-'+'\n')
+        f.write(79*'-'+ '\n%s\n' % str(s).strip() + 79*'-'+'\n')
         f.flush()
 
     # Alias
@@ -634,7 +724,7 @@ class Log:
         f = self.open_log_file or self.open()
         f.write("%04i %s [%s] %s\n" %
                 (errno,_DateTime.now(),self.log_id, errorname))
-        f.write(79*'-'+ '\n%s\n' % _string.join(l,'\n') + 79*'-'+'\n')
+        f.write(79*'-'+ '\n%s\n' % '\n'.join(l) + 79*'-'+'\n')
         f.flush()
 
     def object(self,errno,objname,object,levels=2):
@@ -669,7 +759,7 @@ class Log:
             return
 
         # Write out all complete lines stored in line_buffer
-        lines = _string.split(buffer,'\n')
+        lines = buffer.split('\n')
         if buffer[-1:] != '\n':
             # last line is incomplete
             self.line_buffer = lines[-1]
@@ -682,7 +772,7 @@ class Log:
         id = self.log_id
         fwrite = f.write
         for line in lines:
-            if not _string.strip(line):
+            if not line.strip():
                 # avoid emtpy lines
                 continue
             fwrite("%04i %s [%s] %s\n" % (0,now,id,line))
@@ -709,6 +799,7 @@ class Log:
             s = args[0]
         else:
             s = ''
+
         # go down the traceback
         tb = exc_traceback
         while tb.tb_next is not None:
@@ -722,11 +813,118 @@ class Log:
                  _traceback.tb_lineno(tb),
                  exc_class,exc_inst))
         f.flush()
+
         # Cleanup to avoid circ. refs
         del exc_class,exc_inst,exc_traceback
 
+### Variants of the main Log class #####################################
+
+### ListLog logs to a list instead of a file
+
+class ListFile:
+
+    """ Wraps a list using a file-like interface.
+
+        Just enough to work as log file for mxLog.
+
+    """
+
+    # Log list
+    data = None
+
+    def __init__(self, data, *args):
+        self.data = data
+
+    def write(self, info):
+        self.data.append(info)
+
+    def flush(self):
+        pass
+
+    def close(self):
+        pass
+
+class ListLog(Log):
+
+    """ mxLog variant that logs to a list instead of a file.
+
+    """
+
+    def __init__(self, log_list, *args, **kws):
+
+        self.open_log_file = ListFile(log_list)
+        Log.__init__(self, *args, **kws)
+
+    def close(self):
+        pass
+
+### ThreadLog includes the thread id in the log_id
+
+try:
+    import threading as _threading
+
+except (ImportError, NameError):
+    pass
+
+else:
+
+    class ThreadLog(Log, object):
+
+        """ mxLog variant logging both the PID and thread name of the
+            caller
+
+            It can be used as a common logging facility for multiple
+            threads, since log output from each thread can be identified
+            by thread name.
+
+        """
+        # Custom log_id set via e.g. .setup()
+        _custom_log_id = ''
+
+        def format_log_id(self, log_id=None):
+
+            if log_id is not None:
+                self._custom_log_id = log_id
+
+        def _get_log_id(self):
+
+            if self._custom_log_id:
+                return '%s:%i/%s' % (
+                    self._custom_log_id,
+                    _os.getpid(),
+                    _threading.currentThread().getName())
+            else:
+                return 'p%i/%s' % (
+                    _os.getpid(),
+                    _threading.currentThread().getName())
+
+        def _set_log_id(self, value):
+
+            self._custom_log_id = value
+
+        log_id = property(_get_log_id, _set_log_id)
+
+### LogNothing disabled logging completely
+
+class LogNothing(Log):
+
+    # Don't log anything
+    ignore_level = SYSTEM_LOG_NOTHING
+
+    def setup(self, *args, **kws):
+        Log.setup(*args, **kws)
+        self.ignore_level = SYSTEM_LOG_NOTHING
+
+    def open(self, flags='a'):
+        self.open_log_file = None
+
+    def close(self):
+        pass
+    
+###
+
 # Create a main log object
 if __debug__ and Tools.debugging():
     log = Log(SYSTEM_LOG_EVERYTHING)
 else:
-    log = Log(SYSTEM_INFO+1)
+    log = Log(SYSTEM_INFO + 1)
