@@ -75,15 +75,15 @@
         characters and spanning multiple lines)
 
     Copyright (c) 2000, Marc-Andre Lemburg; All Rights Reserved.
-    Copyright (c) 2000-2010, eGenix.com Software GmbH; All Rights Reserved.
+    Copyright (c) 2000-2013, eGenix.com Software GmbH; All Rights Reserved.
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 
 """
 import copy,re,types,exceptions,sys,os
 import mx.Tools.NewBuiltins
-from mx.Tools import freeze
-import Namespace
+from mx.Tools import freeze, reval
+from mx.Misc import Namespace
 
 ### Globals
 
@@ -160,15 +160,34 @@ class Entry:
     # Default value
     default = ''
 
+    # Allow None as value ?
+    allow_none = 0
+
     # Converters to be tried; order is important the first successful
     # one wins
     converters = (int,float,str)
 
 
-    def __init__(self,default=NotGiven):
+    def __init__(self, default=NotGiven, allow_none=None):
 
+        """ Create an Entry instance.
+
+            default is the default value to use for the Entry in case
+            no value is given in the config file.
+
+            allow_none can be set to 1 to allow an empty string value
+            or a text"none" (including case variants) to result in
+            None being assigned as value.
+
+            If default is None, allow_none automatically get set to 1.
+
+        """
         if default is not NotGiven:
             self.default = default
+        if allow_none is not None:
+            self.allow_none = allow_none
+        if self.default is None:
+            self.allow_none = 1
 
     def parse(self,text,locals,globals):
 
@@ -179,16 +198,23 @@ class Entry:
             
         """
         text = self.pre_process(str(text),locals,globals)
-        for converter in self.converters:
-            try:
-                value = converter(text)
-                break
-            except:
-                lastexc = sys.exc_info()[1]
+        if (self.allow_none and
+            (not text or
+             text.lower() == 'none')):
+            # Use None in case the text is empty or "None"
+            value = None
         else:
-            # Reraise the last exception
-            raise lastexc
-        lastexc = None
+            # Run text through the converters to parse it
+            for converter in self.converters:
+                try:
+                    value = converter(text)
+                    break
+                except:
+                    lastexc = sys.exc_info()[1]
+            else:
+                # Reraise the last exception
+                raise lastexc
+            lastexc = None
         self.value = value = self.post_process(value,locals,globals)
         return value
 
@@ -593,8 +619,15 @@ class EvalEntry(Entry):
 
               eval=eval):
 
-        self.value = eval(text,locals,globals)
-        return self.value
+        if (self.allow_none and
+            (not text or
+             text.lower() == 'none')):
+            # Use None in case the text is empty or "None"
+            value = None
+        else:
+            value = eval(text, locals, globals)
+        self.value = value
+        return value
         
 class SafeEvalEntry(Entry):
 
@@ -602,15 +635,23 @@ class SafeEvalEntry(Entry):
         current locals (the symbols defined in the class
         where the entry is located) and globals.
 
-        Builtins are not available.
+        Builtins are not available and neither are globals passed to
+        the .parse() method.
 
     """
     def parse(self,text,locals,globals,
 
               reval=reval):
 
-        self.value = reval(text,locals,globals)
-        return self.value
+        if (self.allow_none and
+            (not text or
+             text.lower() == 'none')):
+            # Use None in case the text is empty or "None"
+            value = None
+        else:
+            value = reval(text, locals)
+        self.value = value
+        return value
         
 class DictEntry(Entry):
 
@@ -772,7 +813,7 @@ class ConfigFile:
         the file. Section classes are replaced with ConfigSection
         instances.
 
-        After successful reading the file, the configuration
+        After successfully reading the file, the configuration
         information is available through this variable.
 
         Errors are indicated by exceptions of type Error. These always
@@ -788,40 +829,51 @@ class ConfigFile:
         self.template = template
         self.reset()
 
-    def reset(self,
-
-              deepcopy=copy.deepcopy,ClassType=types.ClassType,
-              ModuleType=types.ModuleType):
+    def reset(self):
 
         """ Reset the object to its initial state.
 
-            This initializes self.data to a template instance.
+            This initializes self.data to a template instance and
+            prepares it for reading a config file.
+
+            Note that after running .reset(), the .data entries are
+            Entry instances, not actual values.
 
         """
         self.data = self.template()
 
-    def read(self,file,
+    def read(self, file=None,
 
              deepcopy=copy.deepcopy):
 
         """ Read and parse the open file.
 
+            If file is not given or None, the configuration will be
+            set to all default values.
+
             The configuration is stored in the instance variable
             .data.
 
         """
-        context = {}
+        if not file:
+            # Nothing much to read
+            self.finalize()
+            return
+
+        # Prepare the namespaces
         sectionname = ''
         data = self.data                # Configuration object
         section = data                  # Current section object
         globals = data.__dict__         # Global namespace
         locals = globals                # Local namespace
+
+        # Read file
         lineno = 0
         if _debug:
             print 'Reading config file:'
         while 1:
             line = file.readline()
-            lineno = lineno + 1
+            lineno += 1
             if not line:
                 break
             line = line.strip()
@@ -830,13 +882,13 @@ class ConfigFile:
                 continue
 
             if _debug:
-                print ' read %s' % repr(line)
+                print ' read %r' % line
 
             # Continuation
             current_lineno = lineno
             while line[-1] == '\\':
                 nextline = file.readline()
-                lineno = lineno + 1
+                lineno += 1
                 if not nextline:
                     # EOF: end of continuation
                     line = line[:-1]
@@ -869,9 +921,9 @@ class ConfigFile:
                         section = data.DefaultSection()
                         setattr(data,sectionname,section)
                     else:
-                        raise Error,(current_lineno,
-                                     'unknown section "%s"' % 
-                                     sectionname)
+                        raise Error(current_lineno,
+                                    'unknown section "%s"' % 
+                                    sectionname)
                 #print 'Switched to section:',sectionname
                 locals = section.__dict__
                 continue
@@ -892,22 +944,23 @@ class ConfigFile:
                         entry = deepcopy(section.DefaultAttribute)
                         setattr(section,name,entry)
                     else:
-                        raise Error,(current_lineno,
-                                     'unknown attribute: "%s"' % attrname)
+                        raise Error(current_lineno,
+                                    'unknown attribute: "%s"' % attrname)
                 if not isinstance(entry, Entry):
                     # Since we replace the parsed entries with their parsed
                     # value any occurance of non-Entry instances point
                     # to a duplicate definition of an entry
-                    raise Error,(current_lineno,
-                                 'duplicate attribute definition for "%s"' %
-                                 attrname)
+                    raise Error(current_lineno,
+                                'duplicate attribute definition for "%s"' %
+                                attrname)
                 try:
                     entry.parse(value,locals,globals)
                 except:
-                    raise Error,(current_lineno,
-                                 'invalid attribute value for "%s" (%s: %s)' %
-                                 (attrname,
-                                  sys.exc_info()[0],sys.exc_info()[1]))
+                    raise Error(current_lineno,
+                                'invalid attribute value for "%s" (%s: %s)' %
+                                (attrname,
+                                 sys.exc_info()[0],
+                                 sys.exc_info()[1]))
                 # Replace with parsed value
                 locals[name] = entry.value
                 continue
@@ -918,8 +971,23 @@ class ConfigFile:
                       (current_lineno,
                        repr(line))
 
+        # Finalize the parsed data
+        self.finalize()
+
+    def finalize(self):
+
+        """ Finalize the parsed data in .data.
+
+            This replaces all entries that haven't been set by the
+            configuration file data with default values.
+
+        """
+        sectionname = ''
+        globals = self.data.__dict__    # Global namespace
+        locals = globals                # Local namespace
+
         # Check for missing entries and replace with parsed values
-        for name,obj in globals.items():
+        for name, obj in globals.iteritems():
 
             if isinstance(obj,Entry):
                 if obj.value is not NotGiven:
@@ -929,13 +997,17 @@ class ConfigFile:
                         obj.parse(obj.default,locals,globals)
                         globals[name] = obj.value
                     except:
-                        raise Error,\
-                              (0,
-                               'invalid default attribute '
-                               'value for global "%s" (%s)' %
-                               (name,str(sys.exc_info()[1])))
+                        if _debug:
+                            import traceback; traceback.print_exc()
+                        raise Error(
+                            0,
+                            'invalid default attribute '
+                            'value for global "%s" (%s)' %
+                            (name,
+                             str(sys.exc_info()[1])))
 
             elif isinstance(obj,ConfigSection):
+                # Enter section and switch local namespace
                 locals = obj.__dict__
                 sectionname = name
                 for name,obj in locals.items():
@@ -947,13 +1019,22 @@ class ConfigFile:
                                 obj.parse(obj.default,locals,globals)
                                 locals[name] = obj.value
                             except:
-                                import traceback; traceback.print_exc()
-                                raise Error,\
-                                      (0,
-                                       'invalid default attribute '
-                                       'value for local "%s" (%s)' %
-                                       (name,str(sys.exc_info()[1])))
+                                if _debug:
+                                    import traceback; traceback.print_exc()
+                                raise Error(
+                                    0,
+                                    'invalid default attribute '
+                                    'value for local "%s" (%s)' %
+                                    (name,
+                                     str(sys.exc_info()[1])))
 
+    def read_defaults(self):
+
+        """ Reads an empty file and adjust all settings to their
+            default values.
+
+        """
+        self.finalize()
 
 ### Test
         
@@ -981,6 +1062,7 @@ if __name__ == '__main__':
         class DEF(ConfigSection):
             a = IntegerEntry('0')
             b = EvalEntry('0')
+            b1 = SafeEvalEntry('0')
             c = StringEntry('default value')
             d = RelativePathEntry('$ABC.b','HOME')
             g = IntegerEntry('0')
@@ -1013,7 +1095,10 @@ if __name__ == '__main__':
 
     [DEF]
     a = 2
-    b = a + 3
+    b = sys.maxint # works, since sys is a global
+    #b1 = b - 5
+    #b1 = sys.maxint # raises an exception, since sys is not a local
+    #b1 = None # works, since None is a compiled builtin
     c = $PWD
     d = home
 
