@@ -3,7 +3,7 @@
 """ mysplit - an alternative split implementation (Version 0.1)
     
     Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2008, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
@@ -63,7 +63,6 @@ def mysplitex(text,
 if __name__ == '__main__':
 
     import time
-    from string import split
 
     tries = tuple(range(100000))
     probe = 'abc def ghi jkjkjl asfwer sdfswer sfasgwer svasdwer'
@@ -88,9 +87,9 @@ if __name__ == '__main__':
         l = sp(probe,ws)
     print time.clock() - t,'sec.'
         
-    print 'string.split:',
+    print 'str.split:',
     t = time.clock()
     for i in tries:
-        l = split(probe)
+        l = probe.split()
     print time.clock() - t,'sec.'
         
