@@ -1,11 +1,11 @@
 """ mxTextTools - A tools package for fast text processing.
 
     Copyright (c) 2000, Marc-Andre Lemburg; mailto:mal@lemburg.com
-    Copyright (c) 2000-2009, eGenix.com Software GmbH; mailto:info@egenix.com
+    Copyright (c) 2000-2013, eGenix.com Software GmbH; mailto:info@egenix.com
     See the documentation for further information on copyrights,
     or contact the author. All Rights Reserved.
 """
-import string,types
+import types
 
 #
 # import the C module and the version number
@@ -67,7 +67,7 @@ def format_table(table,i=-1):
             l.append('--> '+format_entry(table,j))
         else:
             l.append('    '+format_entry(table,j))
-    return string.join(l,'\n')+'\n'
+    return '\n'.join(l)+'\n'
 
 def print_tagtable(table):
 
@@ -155,7 +155,7 @@ def word_in_list(l):
     keys = d.keys()
     if len(keys) < 18: # somewhat arbitrary bound
         # fast hint for small sets
-        t.append((None,IsIn,string.join(d.keys(),'')))
+        t.append((None,IsIn,''.join(d.keys())))
         t.append((None,Skip,-1))
     # test groups
     for c, group in d.items():
@@ -176,10 +176,10 @@ def word_in_list(l):
 def replace(text,match,replacement,start=0,stop=None,
 
             SearchObject=TextSearch,join=join,joinlist=joinlist,tag=tag,
-            string_replace=string.replace,type=type,
+            type=type,
             StringType=types.StringType):
 
-    """ A fast replacement for string.replace.
+    """ A fast replacement for str.replace().
     
         match can be given as string or search object.
 
@@ -194,7 +194,7 @@ def replace(text,match,replacement,start=0,stop=None,
         match = so.match
     if stop is None:
         if start == 0 and len(match) < 2:
-            return string_replace(text,match,replacement)
+            return text.replace(match,replacement)
         stop = len(text)
     t = ((text,sWordStart,so,+2),
          # Found something, replace and continue searching
@@ -214,12 +214,12 @@ def _replace2(text,match,replacement,start=0,stop=None,
               join=join,joinlist=joinlist,tag=tag,
               TextSearchType=TextSearchType,TextSearch=TextSearch):
 
-    """ Analogon to string.replace; returns a string with all occurences
+    """ Analogon to str.replace(); returns a string with all occurences
         of match in text[start:stop] replaced by replacement.
        
         This version uses a one entry tag-table and a
         Boyer-Moore-Search-object.  match can be a string or a
-        TextSearch search object.  It's faster than string.replace in
+        TextSearch search object.  It's faster than str.replace() in
         those cases, where the match-string gets long and/or many
         replacements are found; faster meaning from a few percent up
         to many times as fast
@@ -240,7 +240,7 @@ def _replace2(text,match,replacement,start=0,stop=None,
 
 def _replace3(text,match,replacement,
 
-              join=string.join,TextSearch=TextSearch,
+              TextSearch=TextSearch,
               TextSearchType=TextSearchType):
 
     if type(match) is not TextSearchType:
@@ -254,7 +254,7 @@ def _replace3(text,match,replacement,
         l.append(text[x:left] + replacement)
         x = right
     l.append(text[x:])
-    return join(l,'')
+    return ''.join(l)
 
 def _replace4(text,match,replacement,
 
@@ -297,7 +297,7 @@ def find(text,match,start=0,stop=None,
 
          SearchObject=TextSearch):
 
-    """ A faster replacement for string.find().
+    """ A faster replacement for str.find().
 
         Uses a search object for the task. Returns the position of the
         first occurance of match in text[start:stop]. stop defaults to
@@ -330,7 +330,7 @@ def split(text,sep,start=0,stop=None,translate=None,
 
           SearchObject=TextSearch):
 
-    """ A faster replacement for string.split().
+    """ A faster replacement for str.split().
 
         Uses a search object for the task. Returns the result of
         cutting the text[start:stop] string into snippets at every sep
@@ -640,7 +640,6 @@ def _bench(file='mxTextTools/mxTextTools.c'):
         print
         
     text = open(file).read()
-    import string
 
     t = _timer()
     print 'Working on a %i byte string' % len(text)
@@ -655,8 +654,8 @@ def _bench(file='mxTextTools/mxTextTools.c'):
             print 'Replace "%s" with "%s"' % (match,replacement)
             t.start()
             for i in range(100):
-                rtext = string.replace(text,match,replacement)
-            print 'with string.replace:',t.stop(),'sec.'
+                rtext = text.replace(match,replacement)
+            print 'with str.replace():',t.stop(),'sec.'
             t.start()
             for i in range(100):
                 ttext = replace(text,match,replacement)
@@ -697,19 +696,17 @@ def _bench(file='mxTextTools/mxTextTools.c'):
         print '-'*72
         print
 
-        op = string.lower
         t.start()
         for i in range(1000):
-            op(text)
+            text.lower()
         t.stop()
-        print ' string.lower:',t
+        print ' str.lower():',t
 
-        op = string.upper
         t.start()
         for i in range(1000):
-            op(text)
+            text.upper()
         t.stop()
-        print ' string.upper:',t
+        print ' str.upper():',t
 
         op = upper
         t.start()
@@ -726,9 +723,9 @@ def _bench(file='mxTextTools/mxTextTools.c'):
         print ' TextTools.lower:',t
 
         print 'Testing...',
-        ltext = string.lower(text)
+        ltext = text.lower()
         assert ltext == lower(text)
-        utext = string.upper(text)
+        utext = text.upper()
         assert utext == upper(text)
         print 'ok.'
 
@@ -740,12 +737,11 @@ def _bench(file='mxTextTools/mxTextTools.c'):
 
         l = setsplit(text,whitespace_set)
 
-        op = string.join
         t.start()
         for i in range(1000):
-            op(l)
+            ' '.join(l)
         t.stop()
-        print ' string.join:',t
+        print ' str.join():',t
 
         op = join
         t.start()
@@ -754,12 +750,11 @@ def _bench(file='mxTextTools/mxTextTools.c'):
         t.stop()
         print ' TextTools.join:',t
 
-        op = string.join
         t.start()
         for i in range(1000):
-            op(l,' ')
+            '_'.join(l)
         t.stop()
-        print ' string.join with separator:',t
+        print ' str.join() with separator:',t
 
         op = join
         t.start()
@@ -791,12 +786,11 @@ def _bench(file='mxTextTools/mxTextTools.c'):
         print '-'*72
         print
 
-        op = string.split
         t.start()
         for i in range(100):
-            op(text)
+            text.split()
         t.stop()
-        print ' string.split whitespace:',t,'(',len(op(text)),'snippets )'
+        print ' str.split whitespace:',t,'(',len(text.split()),'snippets )'
 
         op = setsplit
         ws = whitespace_set
@@ -806,15 +800,14 @@ def _bench(file='mxTextTools/mxTextTools.c'):
         t.stop()
         print ' TextTools.setsplit whitespace:',t,'(',len(op(text,ws)),'snippets )'
 
-        assert string.split(text) == setsplit(text,ws)
+        assert text.split() == setsplit(text,ws)
 
-        op = string.split
         sep = 'a'
         t.start()
         for i in range(100):
-            op(text,sep)
+            text.split(sep)
         t.stop()
-        print ' string.split at "a":',t,'(',len(op(text,sep)),'snippets )'
+        print ' str.split at "a":',t,'(',len(text.split(sep)),'snippets )'
 
         op = split
         sep = 'a'
@@ -840,15 +833,14 @@ def _bench(file='mxTextTools/mxTextTools.c'):
         t.stop()
         print ' TextTools.setsplit at "a":',t,'(',len(op(text,sep)),'snippets )'
 
-        # Note: string.split and setsplit don't work identically !
+        # Note: str.split and setsplit don't work identically !
 
-        op = string.split
         sep = 'int'
         t.start()
         for i in range(100):
-            op(text,sep)
+            text.split(sep)
         t.stop()
-        print ' string.split at "int":',t,'(',len(op(text,sep)),'snippets )'
+        print ' str.split at "int":',t,'(',len(text.split(sep)),'snippets )'
 
         op = split
         sep = 'int'
@@ -866,13 +858,12 @@ def _bench(file='mxTextTools/mxTextTools.c'):
         t.stop()
         print ' TextTools.setsplit at "i", "n", "t":',t,'(',len(op(text,sep)),'snippets )'
 
-        op = string.split
         sep = 'register'
         t.start()
         for i in range(100):
-            op(text,sep)
+            text.split(sep)
         t.stop()
-        print ' string.split at "register":',t,'(',len(op(text,sep)),'snippets )'
+        print ' str.split at "register":',t,'(',len(text.split(sep)),'snippets )'
 
         op = split
         sep = 'register'
