@@ -39,7 +39,7 @@ routines, timing gadgets and exception helpers.
 END PYTHON-PACKAGE-INFO
 """
 
-import types,sys,time,string,re
+import types,sys,time,re
 
 INDENT = '    ' # string used to indent one level in show,docs,...
 
@@ -120,25 +120,24 @@ def func_sig(func):
     if code.co_flags & 0x0008: # CO_VARKEYWORDS
         args.append('**'+code.co_varnames[callargs])
         callargs = callargs + 1
-    return '%s(%s)' % (fname,string.join(args,', '))
+    return '%s(%s)' % (fname,', '.join(args))
 
 spaces = re.compile('([ ]*)[^ ]')
 
 def showdocstring(doc,level=0,
 
-                  split=re.split,INDENT=INDENT,join=string.join,
-                  strip=string.strip,expandtabs=string.expandtabs,
+                  split=re.split,INDENT=INDENT,
                   spaces=spaces):
 
     try:
-        l = split(strip(doc),'\n\|\r\|\r\n')
+        l = split(doc.strip(),'\n\|\r\|\r\n')
     except:
         #print '%sno doc string available' % (INDENT*level)
         return
     if len(l) > 1:
         # Try to even out the indents
         indent = sys.maxint
-        l = map(expandtabs,l)
+        l = [line.expandtabs() for line in l]
         for i in range(1,len(l)):
             m = spaces.match(l[i])
             if m:
@@ -148,11 +147,11 @@ def showdocstring(doc,level=0,
             else:
                 # Blank line
                 pass
-        l = [strip(l[0])] + map(lambda x,indent=indent: x[indent:],l[1:])
+        l = [l[0].strip()] + map(lambda x,indent=indent: x[indent:],l[1:])
     else:
-        l = map(strip,l)
+        l = [x.strip() for x in l]
     l = map( lambda x,n=level,INDENT=INDENT: INDENT*n + x, l)
-    s = join(l,'\n')+'\n'
+    s = '\n'.join(l)+'\n'
     print s
 
 def showobject(obj,name='',level=0):
@@ -503,13 +502,13 @@ def modules():
     l.sort()
     print 'Loaded modules and packages:'
     for k,v in l:
-        p = string.split(k,'.')
+        p = k.split('.')
         for i in range(len(p)-1):
             p[i] = '   '
-        n = string.join(p,'')
+        n = ''.join(p)
         if v is not None:
             if hasattr(v,'__path__'):
-                print ' %s[%s]' % (string.join(p[:-1],''),p[-1])
+                print ' %s[%s]' % (''.join(p[:-1]),p[-1])
             else:
                 print ' %s' % (n)
 
